import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import Children from '../../children';
import { isNumber, isArray, mix } from '@antv/util';
import getShapeAttrs from '../shape';
import { FunctionComponent, getWorkTag, Shape } from '../workTags';
import computeFlexLayout from './css-layout';
function createMeasure(style, measureText) {
  return function /* width */
  () {
    var text = style.text,
      width = style.width,
      height = style.height;
    var outputWidth = width;
    var outputHeight = height;
    if (!isNumber(width) || !isNumber(height)) {
      var _measureText = measureText(text, style),
        measureWidth = _measureText.width,
        measureHeight = _measureText.height;
      if (!isNumber(width)) {
        outputWidth = measureWidth;
      }
      if (!isNumber(height)) {
        outputHeight = measureHeight;
      }
    }
    return {
      width: outputWidth,
      height: outputHeight
    };
  };
}
function getChildrenLayout(nodeTree) {
  if (!nodeTree) return;
  var left = 0;
  var top = 0;
  var right = 0;
  var bottom = 0;
  var width = 0;
  var height = 0;
  nodeTree.forEach(function (node) {
    var layout = node.layout;
    if (!layout) return;
    left = Math.min(left, layout.left);
    top = Math.min(top, layout.top);
    right = Math.min(right, layout.left + layout.width);
    bottom = Math.min(bottom, layout.top + layout.height);
    width = Math.max(width, layout.width);
    height = Math.max(height, layout.height);
  });
  return {
    left: left,
    top: top,
    right: right,
    bottom: bottom,
    width: width,
    height: height
  };
}
var NodeTree = /*#__PURE__*/function () {
  function NodeTree(node) {
    _classCallCheck(this, NodeTree);
    var className = node.className,
      children = node.children,
      layout = node.layout;
    var nodeChildren = children && children.length ? children.map(function (child) {
      return new NodeTree(child);
    }) : undefined;
    var nodeLayout = layout ? layout : getChildrenLayout(nodeChildren);
    this.children = nodeChildren;
    this.layout = nodeLayout;
    this.className = className;
  }
  _createClass(NodeTree, [{
    key: "getElementsByClassName",
    value: function getElementsByClassName(targetClassName) {
      var result = [];
      var className = this.className,
        children = this.children;
      if (className === targetClassName) {
        result.push(this);
      }
      if (children) {
        children.forEach(function (child) {
          result.push.apply(result, _toConsumableArray(child.getElementsByClassName(targetClassName)));
        });
      }
      return result;
    }
  }]);
  return NodeTree;
}(); // 展开数组
function extendMap(arr, fn) {
  if (!arr) {
    return arr;
  }
  if (!isArray(arr)) {
    return [fn(arr)];
  }
  var newArray = [];
  for (var i = 0; i < arr.length; i++) {
    var element = arr[i];
    if (isArray(element)) {
      newArray = newArray.concat(extendMap(element, fn));
    } else if (element) {
      newArray.push(fn(element));
    }
  }
  return newArray;
}
// 主要是把function节点，全部转换成string标签节点
function renderJSXElement(element, context, updater) {
  if (!element) return element;
  var px2hd = context.px2hd,
    measureText = context.measureText;
  var type = element.type,
    props = element.props;
  var tag = getWorkTag(type);
  // 只处理 function 组件
  if (tag === FunctionComponent) {
    // @ts-ignore
    var newElement = type(element.props, context, updater);
    // return element if type is string
    return renderJSXElement(newElement, context, updater);
  }
  var className = props.className,
    _props$style = props.style,
    customStyle = _props$style === void 0 ? {} : _props$style,
    attrs = props.attrs,
    newChildren = props.children;
  var style = px2hd(_objectSpread(_objectSpread({}, customStyle), attrs));
  // 文本需要计算文本的宽高来进行flex布局
  if (type === 'text') {
    style.measure = createMeasure(style, measureText);
  }
  // render children first
  var nextChildren = newChildren ? Children.toArray(newChildren).map(function (child) {
    return renderJSXElement(child, context, updater);
  }) : [];
  return {
    type: type,
    className: className,
    children: nextChildren.filter(Boolean),
    style: style
  };
}
// 计算布局
function computeLayout(component, newChildren) {
  var context = component.context,
    updater = component.updater;
  var nodeTree = renderJSXElement(newChildren, context, updater);
  computeFlexLayout(nodeTree);
  // 构造一个 NodeTree, 方便外部使用
  return new NodeTree(nodeTree);
}
// 创建布局的计算树
function createNodeTree(vNode) {
  var type = vNode.type,
    style = vNode.style,
    context = vNode.context,
    vNodeChildren = vNode.children;
  var measureText = context.measureText;
  var children = extendMap(vNodeChildren, function (child) {
    return createNodeTree(child);
  });
  // 文本需要计算文本的宽高来进行flex布局
  if (type === 'text') {
    style.measure = createMeasure(style, measureText);
  }
  return {
    type: type,
    style: style,
    children: children,
    // 保留对 vNode 的引用，用于把布局结果回填
    vNode: vNode
  };
}
function fillElementLayout(node) {
  var type = node.type,
    style = node.style,
    vNode = node.vNode,
    children = node.children,
    layout = node.layout;
  var attrs = getShapeAttrs(type, layout);
  if (style.measure) {
    delete style.measure;
  }
  // 更新布局和样式
  vNode.layout = layout;
  vNode.style = _objectSpread(_objectSpread({}, attrs), style);
  if (children && children.length) {
    for (var i = 0, len = children.length; i < len; i++) {
      var child = children[i];
      var _vNode = child.vNode;
      var tag = _vNode.tag,
        childStyle = _vNode.style;
      if (tag !== Shape) {
        if (layout) {
          mix(childStyle, {
            left: layout.left,
            top: layout.top,
            width: layout.width,
            height: layout.height
          });
        } else {
          mix(childStyle, {
            left: style.left,
            top: style.top,
            width: style.width,
            height: style.height
          });
        }
      }
      fillElementLayout(child);
    }
  }
}
export { computeLayout, createNodeTree, computeFlexLayout, fillElementLayout };