import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { isArray, isNil } from '@antv/util';
function cloneElement(element, props) {
  if (!element) return element;
  return _objectSpread(_objectSpread({}, element), {}, {
    props: _objectSpread(_objectSpread({}, element.props), props)
  });
}
function map(children, fn) {
  if (!children) {
    return fn(children);
  }
  if (isArray(children)) {
    return children.map(function (child) {
      return map(child, fn);
    });
  }
  return fn(children);
}
function compareArray(nextElements, lastElements, callback) {
  var keyed = {};
  var nextLength = nextElements.length;
  var lastLength = lastElements.length;
  for (var i = 0, len = lastLength; i < len; i++) {
    var element = lastElements[i];
    if (element && !isNil(element.key)) {
      var key = element.key;
      keyed[key] = element;
    }
  }
  var result = [];
  // 比较元素
  for (var _i = 0, _len = nextLength; _i < _len; _i++) {
    var _lastElement;
    var _element = nextElements[_i];
    if (!_element) {
      continue;
    }
    var _key = _element.key;
    var lastElement = void 0;
    // 有key值定义
    if (!isNil(_element.key)) {
      lastElement = keyed[_key];
      if (lastElement) delete keyed[_key];
    } else {
      // 取相同位置的元素
      lastElement = lastElements[_i];
    }
    // 没有直接返回
    if (!lastElement) {
      result.push(compare(_element, null, callback));
      continue;
    }
    // 如果 lastElement 已经被处理过, next 处理成新增
    if ((_lastElement = lastElement) === null || _lastElement === void 0 ? void 0 : _lastElement.__processed) {
      result.push(compare(_element, null, callback));
      continue;
    }
    // 标记 element 已经被处理过
    lastElement.__processed = true;
    result.push(compare(_element, lastElement, callback));
  }
  // 处理 lastElements 里面还未被处理的元素
  for (var _i2 = 0, _len2 = lastLength; _i2 < _len2; _i2++) {
    var _lastElement2 = lastElements[_i2];
    if (!_lastElement2) {
      continue;
    }
    if (!(_lastElement2 === null || _lastElement2 === void 0 ? void 0 : _lastElement2.__processed)) {
      result.push(compare(null, _lastElement2, callback));
    } else {
      delete _lastElement2.__processed;
    }
  }
  return result;
}
// 比较2棵树
function compare(nextElement, lastElement, callback) {
  // 有一个为空
  if (!nextElement || !lastElement) {
    return callback(nextElement, lastElement);
  }
  if (isArray(nextElement) || isArray(lastElement)) {
    var nextElementArray = isArray(nextElement) ? nextElement : [nextElement];
    var lastElementArray = isArray(lastElement) ? lastElement : [lastElement];
    return compareArray(nextElementArray, lastElementArray, callback);
  }
  return callback(nextElement, lastElement);
}
function toArray(element) {
  if (!element) {
    return element;
  }
  if (!isArray(element)) {
    return [element];
  }
  var newArray = [];
  for (var i = 0, len = element.length; i < len; i++) {
    var item = element[i];
    if (isArray(item)) {
      // @ts-ignore
      newArray = newArray.concat(toArray(item));
    } else {
      newArray.push(item);
    }
  }
  return newArray;
}
var Children = {
  cloneElement: cloneElement,
  map: map,
  toArray: toArray,
  compare: compare
};
export default Children;