"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createAnimation = createAnimation;
var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));
var _children = _interopRequireDefault(require("../../children"));
var _gLite = require("@antv/g-lite");
var _util = require("@antv/util");
var _createShape = require("./createShape");
function applyStyle(shape, style) {
  if (!style) return;
  Object.keys(style).forEach(function (key) {
    // 特殊处理 clip
    if (key === 'clip') {
      var clip = style.clip;
      var clipConfig = (0, _util.isFunction)(clip) ? clip(style) : clip;
      if (clipConfig) {
        var type = clipConfig.type,
          _style = clipConfig.style;
        var clipShape = (0, _createShape.createShape)(type, {
          style: _style
        });
        shape.setAttribute('clipPath', clipShape);
      }
    } else {
      shape.setAttribute(key, style[key]);
    }
  });
}
function appearAnimation(vNode) {
  return _children.default.map(vNode, function (node) {
    if (!node) return;
    var shape = node.shape,
      style = node.style,
      children = node.children,
      animate = node.animate,
      props = node.props,
      animator = node.animator;
    // 有叶子节点，先执行叶子节点
    var childrenAnimation = children ? createAnimation(node, children, null) : null;
    // 不需要执行动画
    if (animate === false) {
      applyStyle(shape, style);
      return null;
    }
    var animation = props.animation;
    var animationEffect = animation ? animation.appear : null;
    animator.reset();
    if (!animationEffect) {
      // 没有动画直接应用样式
      applyStyle(shape, style);
    }
    // 如果当前和子元素都没有动画，直接返回
    if (!animationEffect && !(childrenAnimation && childrenAnimation.length)) {
      return null;
    }
    // 子元素动画
    if (childrenAnimation && childrenAnimation.length) {
      animator.children = childrenAnimation;
    }
    // 图形有动画
    if (animationEffect) {
      var _animationEffect$star = animationEffect.start,
        start = _animationEffect$star === void 0 ? {} : _animationEffect$star,
        end = animationEffect.end;
      var endStyle = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, style), end);
      animator.animate(shape, start, endStyle, animationEffect);
    }
    return animator;
  });
}
function updateAnimation(nextNode, lastNode) {
  var nextType = nextNode.type,
    nextStyle = nextNode.style,
    nextChildren = nextNode.children,
    nextProps = nextNode.props,
    nextShape = nextNode.shape,
    animator = nextNode.animator,
    animate = nextNode.animate;
  var lastType = lastNode.type,
    lastStyle = lastNode.style,
    lastChildren = lastNode.children,
    lastShape = lastNode.shape;
  // 先处理叶子节点
  var childrenAnimation = createAnimation(nextNode, nextChildren, lastChildren);
  // 清除之前的样式
  var resetStyle = lastStyle ? Object.keys(lastStyle).reduce(function (prev, cur) {
    prev[cur] = '';
    return prev;
  }, {}) : null;
  // 需要更新的样式
  var style = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, resetStyle), nextStyle);
  // 没有动画直接应用样式
  if (animate === false) {
    applyStyle(nextShape, style);
    return null;
  }
  var animation = nextProps.animation;
  animator.reset();
  // 子元素动画
  if (childrenAnimation && childrenAnimation.length) {
    animator.children = childrenAnimation;
  }
  var animationEffect = animation ? animation.update : null;
  // 没有动画直接应用样式
  if (!animationEffect) {
    applyStyle(nextShape, style);
    // 如果也没有子元素动画，直接返回
    if (!childrenAnimation || !childrenAnimation.length) {
      return null;
    }
  }
  // 自身动画
  if ((0, _util.isString)(nextType) && (0, _util.isString)(lastType)) {
    if (animationEffect) {
      // 图形属性动画
      if (nextType === lastType) {
        var start = animationEffect.start,
          end = animationEffect.end;
        var startStyle = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, lastStyle), start);
        var endStyle = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, style), end);
        animator.animate(nextShape, startStyle, endStyle, animationEffect);
      } else {
        // 形变动画
        var _start = animationEffect.start,
          _end = animationEffect.end,
          _animationEffect$prop = animationEffect.property,
          property = _animationEffect$prop === void 0 ? [] : _animationEffect$prop;
        var lastPath = (0, _gLite.convertToPath)(lastShape);
        var nextPath = (0, _gLite.convertToPath)(nextShape);
        var _startStyle = (0, _objectSpread2.default)((0, _objectSpread2.default)((0, _objectSpread2.default)({}, lastStyle), _start), {}, {
          path: lastPath
        });
        var _endStyle = (0, _objectSpread2.default)((0, _objectSpread2.default)((0, _objectSpread2.default)({}, nextStyle), _end), {}, {
          path: nextPath
        });
        var pathShape = (0, _createShape.createShape)('path', {
          style: (0, _objectSpread2.default)((0, _objectSpread2.default)({}, _startStyle), {}, {
            path: ''
          })
        });
        lastShape.replaceWith(pathShape);
        animator.animate(pathShape, _startStyle, _endStyle, (0, _objectSpread2.default)((0, _objectSpread2.default)({}, animationEffect), {}, {
          property: ['path'].concat(property)
        }));
        animator.once('end', function () {
          applyStyle(nextShape, _endStyle);
          pathShape.replaceWith(nextShape);
        });
      }
    }
  }
  return animator;
}
function destroyAnimation(vNode) {
  if (!vNode) return null;
  var shape = vNode.shape,
    children = vNode.children,
    animate = vNode.animate,
    style = vNode.style,
    props = vNode.props,
    animator = vNode.animator;
  // 先处理叶子节点
  var childrenAnimation = children ?
  // @ts-ignore
  _children.default.toArray(children).map(function (child) {
    return destroyAnimation(child);
  }).filter(Boolean) : null;
  if (animate === false) {
    shape.remove();
    return null;
  }
  var animation = props.animation;
  var animationEffect = animation ? animation.leave : null;
  // 重置
  animator.reset();
  // 没有叶子节点的动画， 直接删除
  if (!(childrenAnimation && childrenAnimation.length) && !animationEffect) {
    shape.remove();
    return null;
  }
  if (childrenAnimation && childrenAnimation.length) {
    animator.children = childrenAnimation;
  }
  // 图形有动画
  if (animationEffect) {
    var start = animationEffect.start,
      _animationEffect$end = animationEffect.end,
      end = _animationEffect$end === void 0 ? {} : _animationEffect$end;
    var startStyle = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, style), start);
    var endStyle = end;
    animator.animate(shape, startStyle, endStyle, animationEffect);
  }
  animator.once('end', function () {
    shape.remove();
  });
  return animator;
}
function createAnimator(nextNode, lastNode) {
  if (!nextNode && !lastNode) {
    return;
  }
  // delete 动画
  if (!nextNode && lastNode) {
    return destroyAnimation(lastNode);
  }
  // appear 动画
  if (nextNode && !lastNode) {
    return appearAnimation(nextNode);
  }
  // update 动画
  return updateAnimation(nextNode, lastNode);
}
function createAnimation(parent, nextChildren, lastChildren) {
  var parentShape = parent.shape;
  var nextSibling = parentShape.firstChild;
  var childrenAnimation = [];
  _children.default.compare(nextChildren, lastChildren, function (nextNode, lastNode) {
    var animator = createAnimator(nextNode, lastNode);
    // 更新文档流
    if (nextNode && !lastNode) {
      _children.default.map(nextNode, function (node) {
        if (!node) return;
        var shape = node.shape;
        if (nextSibling) {
          parentShape.insertBefore(shape, nextSibling);
        } else {
          parentShape.appendChild(shape);
        }
      });
    } else if (nextNode && lastNode) {
      var shape = nextNode.shape;
      nextSibling = shape.nextSibling;
    }
    if (animator) {
      _children.default.map(animator, function (item) {
        if (!item) return;
        childrenAnimation.push(item);
      });
    }
    return animator;
  });
  return childrenAnimation;
}