"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "computeLayout", {
  enumerable: true,
  get: function get() {
    return _computeLayout.computeLayout;
  }
});
exports.render = render;
exports.renderChildren = renderChildren;
exports.updateComponents = updateComponents;
var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));
var _util = require("@antv/util");
var _component = _interopRequireDefault(require("../../component"));
var _children = _interopRequireDefault(require("../../children"));
var _createShape = require("../render/createShape");
var _gLite = require("@antv/g-lite");
var _equal = _interopRequireDefault(require("../equal"));
var _animation = require("./animation");
var _animator = _interopRequireDefault(require("./animator"));
var _workTags = require("../workTags");
var _computeLayout = require("./computeLayout");
function getStyle(tagType, props, context) {
  var _props$style = props.style,
    customStyle = _props$style === void 0 ? {} : _props$style,
    attrs = props.attrs,
    zIndex = props.zIndex;
  if (tagType === _workTags.Shape) {
    return context.px2hd((0, _objectSpread2.default)((0, _objectSpread2.default)({}, customStyle), attrs));
  }
  if ((0, _util.isNumber)(zIndex)) {
    return {
      zIndex: zIndex
    };
  }
  return {};
}
function createVNode(parent, vNode) {
  var context = parent.context,
    updater = parent.updater,
    parentAnimate = parent.animate;
  var ref = vNode.ref,
    type = vNode.type,
    props = vNode.props;
  var animate = props.animate,
    transformFrom = props.transformFrom;
  var tag = (0, _workTags.getWorkTag)(type);
  var animator = new _animator.default();
  var style = getStyle(tag, props, context);
  vNode.tag = tag;
  vNode.style = style;
  vNode.context = context;
  vNode.updater = updater;
  vNode.animate = (0, _util.isBoolean)(animate) ? animate : parentAnimate;
  vNode.animator = animator;
  // shape 标签
  if (tag === _workTags.Shape) {
    var shape = (0, _createShape.createShape)(type, (0, _objectSpread2.default)((0, _objectSpread2.default)({}, props), {}, {
      style: style
    }));
    if (ref) {
      ref.current = shape;
    }
    // @ts-ignore
    shape._vNode = vNode; // shape 保留 vNode 的引用
    vNode.shape = shape;
  } else {
    // 组件
    var component;
    if (tag === _workTags.ClassComponent) {
      // @ts-ignore
      component = new type(props, context, updater);
    } else {
      component = new _component.default(props, context, updater);
      component.render = function () {
        // @ts-ignore
        return type(this.props, context, updater);
      };
    }
    var group = new _gLite.Group();
    component.container = group;
    // 设置ref
    if (ref) {
      ref.current = component;
    }
    component.context = context;
    component.updater = updater;
    component.animator = animator;
    component._vNode = vNode;
    vNode.shape = group;
    vNode.component = component;
  }
  if (transformFrom && transformFrom.current) {
    var transformVNode = transformFrom.current._vNode;
    vNode.children = transformVNode.children;
    transformVNode.children = null;
  }
  return vNode;
}
function updateVNode(parent, nextNode, lastNode) {
  var context = parent.context,
    updater = parent.updater,
    parentAnimate = parent.animate;
  var tag = lastNode.tag,
    animator = lastNode.animator,
    component = lastNode.component,
    shape = lastNode.shape,
    children = lastNode.children;
  var props = nextNode.props;
  var animate = props.animate;
  nextNode.tag = tag;
  nextNode.context = context;
  nextNode.updater = updater;
  nextNode.component = component;
  nextNode.shape = shape;
  nextNode.parent = parent;
  nextNode.children = children;
  nextNode.animate = (0, _util.isBoolean)(animate) ? animate : parentAnimate;
  nextNode.animator = animator;
  nextNode.style = getStyle(tag, props, context);
  return nextNode;
}
function createElement(parent, element) {
  return _children.default.map(element, function (el) {
    if (!el) return el;
    return createVNode(parent, el);
  });
}
function destroyElement(vNode) {
  _children.default.map(vNode, function (node) {
    if (!node) return;
    var component = node.component,
      children = node.children;
    if (component) {
      component.willUnmount();
      destroyElement(children);
      component.didUnmount();
      component.destroy();
    }
  });
}
function updateElement(parent, nextElement, lastElement) {
  var nextType = nextElement.type,
    nextProps = nextElement.props;
  var lastType = lastElement.type,
    lastProps = lastElement.props;
  if (nextType === lastType) {
    var _nextVNode = updateVNode(parent, nextElement, lastElement);
    // props 无变化 和 context 都无变化
    if ((0, _equal.default)(nextProps, lastProps) && parent.context === lastElement.context) {
      return null;
    }
    return _nextVNode;
  }
  var nextVNode = createVNode(parent, nextElement);
  destroyElement(lastElement);
  return nextVNode;
}
function diffElement(parent, nextElement, lastElement) {
  if (!nextElement && !lastElement) {
    return null;
  }
  // 删除
  if (!nextElement && lastElement) {
    destroyElement(lastElement);
    return null;
  }
  // 新建
  if (nextElement && !lastElement) {
    return createElement(parent, nextElement);
  }
  // 更新
  // @ts-ignore
  return updateElement(parent, nextElement, lastElement);
}
function renderComponentNodes(componentNodes) {
  if (!componentNodes || !componentNodes.length) {
    return;
  }
  // 1. shouldUpdate & willReceiveProps
  var shouldProcessChildren = componentNodes.filter(function (node) {
    var component = node.component,
      props = node.props,
      context = node.context,
      style = node.style;
    component.style = style;
    // 新创建的 component
    if (!component.children) return true;
    // 不需要更新
    if (component.shouldUpdate(props) === false) {
      return false;
    }
    component.willReceiveProps(props, context);
    component.props = props;
    component.context = context;
    return true;
  });
  if (!shouldProcessChildren.length) {
    return;
  }
  // 2. willMount / willUpdate
  shouldProcessChildren.forEach(function (child) {
    var component = child.component;
    if (!component.children) {
      component.willMount();
    } else {
      component.willUpdate();
    }
  });
  // 3. render
  shouldProcessChildren.forEach(function (child) {
    var component = child.component,
      children = child.children;
    var mount = !component.children;
    var newChildren = component.render();
    renderChildren(child, newChildren, children);
    if (mount) {
      component.didMount();
    } else {
      component.didUpdate();
    }
  });
}
function renderVNode(node, nextChildren, lastChildren) {
  var component = node.component;
  // 设置新的 children
  node.children = nextChildren;
  // 如果是组件，需要同时更新组件的 children
  // 等同于 node.tag === ClassComponent || node.tag === FunctionComponent
  if (component) {
    component.children = nextChildren;
  }
  var componentNodeChildren = [];
  _children.default.compare(
  // @ts-ignore
  nextChildren,
  // @ts-ignore
  lastChildren, function (next, last) {
    var element = diffElement(node, next, last);
    _children.default.map(element, function (child) {
      if (!child) return;
      var tag = child.tag,
        childProps = child.props,
        component = child.component,
        childLastChildren = child.children;
      var childrenNode = [];
      if (tag === _workTags.Shape) {
        childrenNode = renderVNode(child, childProps.children, childLastChildren);
      } else if (tag === _workTags.FunctionComponent) {
        // FunctionComponent 直接更新 props
        component.props = childProps;
        childrenNode = renderVNode(child, component.render(), childLastChildren);
      } else {
        childrenNode = [child];
      }
      componentNodeChildren = componentNodeChildren.concat(childrenNode);
    });
  });
  return componentNodeChildren;
}
function renderChildren(parent, nextChildren, lastChildren) {
  // 返回的都是 classComponent 的节点
  var componentNodeChildren = renderVNode(parent, nextChildren, lastChildren);
  // 计算 flex 布局
  var nodeTree = (0, _computeLayout.createNodeTree)(parent);
  (0, _computeLayout.computeFlexLayout)(nodeTree);
  (0, _computeLayout.fillElementLayout)(nodeTree);
  var newChildren = parent.children;
  if (!componentNodeChildren.length) {
    return newChildren;
  }
  renderComponentNodes(componentNodeChildren);
  return newChildren;
}
function render(vNode) {
  var lastChildren = vNode.children,
    props = vNode.props;
  var nextChildren = props.children;
  // render 节点
  var children = renderChildren(vNode, nextChildren, lastChildren);
  // vNode.children = children;
  // 创建动画
  var childrenAnimation = (0, _animation.createAnimation)(vNode, children, lastChildren);
  // 执行动画
  if (childrenAnimation.length) {
    childrenAnimation.forEach(function (animator) {
      animator.play();
    });
  }
}
// setState 触发的更新
function updateComponents(components) {
  if (!components.length) return;
  components.forEach(function (component) {
    var node = component._vNode,
      lastChildren = component.children,
      props = component.props,
      animator = component.animator;
    // 是否需要更新
    if (component.shouldUpdate(props) === false) {
      return false;
    }
    component.willUpdate();
    var newChildren = component.render();
    var nextChildren = renderChildren(node, newChildren, lastChildren);
    // 更新 children
    component.children = nextChildren;
    node.children = nextChildren;
    component.didUpdate();
    // 创建动画
    var childrenAnimation = (0, _animation.createAnimation)(node, nextChildren, lastChildren);
    if (childrenAnimation.length) {
      animator.children = childrenAnimation;
    }
    // 执行动画
    animator.play();
  });
}